@@ -5,6 +5,14 @@
 
 const JL_MAX_TAGS = 64 # see `enum jl_small_typeof_tags` in julia.h
 
+function unbind(@nospecialize(val))
+   if val isa Core.Binding
+       return val.value
+   else
+       return val
+   end
+end
+
 function absint(@nospecialize(arg::LLVM.Value), partial::Bool = false, istracked::Bool=false, typetag::Bool=false)::Tuple{Bool, Any}
     if (value_type(arg) == LLVM.PointerType(LLVM.StructType(LLVMType[]), Tracked)) || (value_type(arg) == LLVM.PointerType(LLVM.StructType(LLVMType[]), Derived)) || istracked
         ce, _ = get_base_and_offset(arg; offsetAllowed = false, inttoptr = true)
@@ -20,6 +28,7 @@ function absint(@nospecialize(arg::LLVM.Value), partial::Bool = false, istracked
                     return (true, v)
                 end
             end
+	    @assert !startswith(gname, "ejl_inserted") "Could not find ejl_inserted variable in map $gname"
         end
         if isa(ce, LLVM.LoadInst)
             gv = operands(ce)[1]
@@ -54,6 +63,7 @@ function absint(@nospecialize(arg::LLVM.Value), partial::Bool = false, istracked
             return (true, val)
         end
     end
+
     if isa(arg, ConstantExpr)
         if opcode(arg) == LLVM.API.LLVMAddrSpaceCast || opcode(arg) == LLVM.API.LLVMBitCast
             return absint(operands(arg)[1], partial, false, typetag)
@@ -197,6 +207,7 @@ function absint(@nospecialize(arg::LLVM.Value), partial::Bool = false, istracked
         typ = Base.unsafe_pointer_to_objref(ptr)
         return (true, typ)
     end
+
     return (false, nothing)
 end
 
@@ -362,7 +373,7 @@ function abs_typeof(
             end
             for (k, v) in JuliaEnzymeNameMap
                 if gname == "ejl_" * k
-                    return (true, Core.Typeof(v), GPUCompiler.BITS_REF)
+		    return (true, Core.Typeof(unbind(v)), GPUCompiler.BITS_REF)
                 end
             end
         end
@@ -391,6 +402,7 @@ function abs_typeof(
             return (true, Core.Typeof(val), GPUCompiler.BITS_REF)
         end
     end
+
     if isa(arg, ConstantExpr)
         if opcode(arg) == LLVM.API.LLVMAddrSpaceCast || opcode(arg) == LLVM.API.LLVMBitCast
             return abs_typeof(operands(arg)[1], partial, seenphis)
@@ -452,12 +464,14 @@ function abs_typeof(
                 nm == "jl_gc_alloc_typed" ||
                 nm == "ijl_gc_alloc_typed"
             vals = absint(operands(arg)[3], partial, false, #=typetag=#true)
+	    @assert !(vals[2] isa Core.Binding)
             return (vals[1], vals[2], vals[1] ? GPUCompiler.BITS_REF : nothing)
         end
         # Type tag is arg 3
         if nm == "jl_alloc_genericmemory_unchecked" ||
 		nm == "ijl_alloc_genericmemory_unchecked"
 	    vals = absint(operands(arg)[3], partial, true, #=typetag=#true)
+	    @assert !(vals[2] isa Core.Binding)
             return (vals[1], vals[2], vals[1] ? GPUCompiler.MUT_REF : nothing)
         end
         # Type tag is arg 1
@@ -472,11 +486,13 @@ function abs_typeof(
                 nm == "jl_alloc_genericmemory" ||
                 nm == "ijl_alloc_genericmemory"
             vals = absint(operands(arg)[1], partial, false, #=typetag=#true)
+	    @assert !(vals[2] isa Core.Binding)
             return (vals[1], vals[2], vals[1] ? GPUCompiler.MUT_REF : nothing)
         end
 
         if nm == "jl_new_structt" || nm == "ijl_new_structt"
             vals = absint(operands(arg)[1], partial, false, #=typetag=#true)
+	    @assert !(vals[2] isa Core.Binding)
             return (vals[1], vals[2], vals[1] ? GPUCompiler.MUT_REF : nothing)
         end
 
@@ -495,6 +511,7 @@ function abs_typeof(
             if nm == "jl_new_structv" || nm == "ijl_new_structv"
                 @assert index == 2
                 vals = absint(operands(arg)[index], partial, false, #=typetag=#true)
+	    	@assert !(vals[2] isa Core.Binding)
                 return (vals[1], vals[2], vals[1] ? GPUCompiler.MUT_REF : nothing)
             end
 
@@ -528,9 +545,11 @@ function abs_typeof(
             if nm == "jl_f__apply_iterate" || nm == "ijl_f__apply_iterate"
                 index += 1
                 legal, iterfn = absint(operands(arg)[index])
+	    	iterfn = unbind(iterfn)
                 index += 1
                 if legal && iterfn == Base.iterate
                     legal0, combfn = absint(operands(arg)[index])
+		    combfn = unbind(combfn)
                     index += 1
                     if legal0 && combfn == Core.apply_type && partial
                         return (true, Type, GPUCompiler.BITS_REF)
@@ -868,6 +887,7 @@ function abs_typeof(
 
     legal, val = absint(arg, partial)
     if legal
+	val = unbind(val)
         return (true, Core.Typeof(val), GPUCompiler.BITS_REF)
     end
     return (false, nothing, nothing)
